package sgd.and03.sternzeichen

import android.view.View
import android.widget.NumberPicker
import androidx.test.core.app.ActivityScenario
import androidx.test.espresso.Espresso.onData
import androidx.test.espresso.Espresso.onView
import androidx.test.espresso.NoMatchingViewException
import androidx.test.espresso.UiController
import androidx.test.espresso.ViewAction
import androidx.test.espresso.ViewAssertion
import androidx.test.espresso.action.ViewActions.click
import androidx.test.espresso.assertion.ViewAssertions.matches
import androidx.test.espresso.matcher.ViewMatchers
import androidx.test.espresso.matcher.ViewMatchers.withId
import androidx.test.espresso.matcher.ViewMatchers.withText
import androidx.test.ext.junit.rules.ActivityScenarioRule
import androidx.test.ext.junit.runners.AndroidJUnit4
import androidx.test.platform.app.InstrumentationRegistry
import org.hamcrest.CoreMatchers.`is`
import org.hamcrest.Matcher
import org.junit.Assert.*
import org.junit.Rule
import org.junit.Test
import org.junit.runner.RunWith


/**
 * Instrumented test, which will execute on an Android device.
 *
 * See [testing documentation](http://d.android.com/tools/testing).
 */
@RunWith(AndroidJUnit4::class)
class ExampleInstrumentedTest {
    //@Rule
    //doku veraltet
    // in build.gradle unbedingt die nach doku hinzugefügte dependency entfernen
    @get:Rule
    val activityRule = ActivityScenarioRule(MainActivity::class.java)


    @Test
    fun textView_atStart() {
        onView(withId(R.id.textview_ausgabe)).check(matches(withText( "Wähle Deinen Geburtstag")))
    }

    @Test
    fun textView_at1Januar() {
        onView(withId(R.id.button_OK)).perform(click())
        onView(withId(R.id.textview_ausgabe)).check(matches(withText( "Dein Sternzeichen ist Steinbock")))
    }

    @Test
    fun textView_at1Dezember() {
        onView(withId(R.id.spinner_monat)).perform(click())
        onData(`is`("Dezember")).perform(click())
        onView(withId(R.id.button_OK)).perform(click())
        onView(withId(R.id.textview_ausgabe)).check(matches(withText( "Dein Sternzeichen ist Schütze"))
        )
    }

    fun setValue(value: Int): ViewAction {
        return object: ViewAction {
            override fun perform(uiController: UiController, view: View) {
                (view as NumberPicker).value = value
            }
            override fun getDescription(): String {
                return "Set the passed value into the NumberPicker";
            }
            override fun getConstraints(): Matcher<View> {
                return ViewMatchers.isAssignableFrom(NumberPicker::class.java)
            }
        }
    }

    @Test
    fun textView_at31Dezember() {
        onView(withId(R.id.spinner_monat)).perform(click())
        onData(`is`("Dezember")).perform(click())
        onView(withId(R.id.numberpicker_tag)).perform(setValue(31))
        onView(withId(R.id.button_OK)).perform(click())
        onView(withId(R.id.textview_ausgabe)).check(matches(withText( "Dein Sternzeichen ist Steinbock")))
    }

    @Test
    fun textView_at25Juni() {
        onView(withId(R.id.spinner_monat)).perform(click())
        onData(`is`("Juni")).perform(click())
        onView(withId(R.id.numberpicker_tag)).perform(setValue(25))
        onView(withId(R.id.button_OK)).perform(click())
        onView(withId(R.id.textview_ausgabe)).check(matches(withText( "Dein Sternzeichen ist Krebs")))
    }

    fun testMaxValue(maxValue: Int): ViewAssertion {
        return object : ViewAssertion {
            override fun check(view: View, noViewFoundException: NoMatchingViewException?) {
                if (maxValue != (view as NumberPicker).maxValue)
                    throw junit.framework.AssertionFailedError()
            }
        }
    }
    @Test
    fun numberPicker_maxValueFebruar() {
        onView(withId(R.id.spinner_monat)).perform(click())
        onData(`is`("Februar")).perform(click())
        onView(withId(R.id.numberpicker_tag)).check(testMaxValue(29))
    }
}


class ActivityTestRule(java: Class<MainActivity>) {

}
